'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var queryString = require('query-string');
var history = require('history');
var React = require('react');
var React__default = _interopDefault(React);
var reactRedux = require('react-redux');

function locationToRoute (ref) {
    var pathname = ref.pathname;
    var search = ref.search;

    var path = pathname.split('/');
    if (path[0] === '') { path.shift(); }
    if (path[path.length - 1] === '') { path.pop(); }

    var query = queryString.parse(search);
    return { path: path, query: query }
}

function routeToLocation (ref) {
    var path = ref.path;
    var query = ref.query;

    var pathname = '/' + path.join('/');
    var queryString$$1 = queryString.stringify(query);
    var search = queryString$$1 ? ('?' + queryString$$1) : '';
    return { pathname: pathname, search: search }
}

function routeToURL (route) {
    var ref = routeToLocation(route);
    var pathname = ref.pathname;
    var search = ref.search;
    return pathname + search
}

function hashRouteToURL (route) {
    return '#' + routeToURL(route)
}

function createRouteHandler (ref) {
    var history$$1 = ref.history;
    var onChange = ref.onChange;
    var location = ref.location;

    var setRouteFromLocation = function (location, action) {
        if (!action || action === 'POP') {
            onChange(locationToRoute(location));
        }
    };

    var prevLocation = {};
    var setLocationFromRoute = function (route) {
        var location = routeToLocation(route);
        if (locationEq(prevLocation, location)) { return }
        prevLocation = location;
        history$$1.push(location);
    };

    setRouteFromLocation(location);
    history$$1.listen(setRouteFromLocation);
    return setLocationFromRoute
}

function locationEq (a, b) {
    return a.pathname === b.pathname && a.search === b.search
}

function createRouterMiddleware (params) {
    var history$$1 = params.history;
    var location = params.location;
    var selectRoute = params.selectRoute;
    var onChange = params.onChange;

    return function (ref) {
        var dispatch = ref.dispatch;
        var getState = ref.getState;

        var routeHandler = createRouteHandler({
            history: history$$1 || history.createBrowserHistory(),
            location: location || window.location,
            onChange: function (route) { return dispatch(onChange(route)); },
        });

        return function (next) { return function (action) {
            var value = next(action);

            routeHandler(selectRoute(getState()));

            return value
        }; }
    }
}

function createHashRouterMiddleware (ref) {
    var selectRoute = ref.selectRoute;
    var onChange = ref.onChange;

    return createRouterMiddleware({
        history: history.createHashHistory(),
        location: {
            pathname: window.location.hash.replace('#', '') || '/',
            search: window.location.search,
        },
        selectRoute: selectRoute,
        onChange: onChange,
    })
}

var CONTEXT = 'redux-antirouter/Link/getNextRoute';

var h = React__default.createElement;

var LinkProvider = (function (superclass) {
    function LinkProvider (props) {
        superclass.call(this, props);
        this.initProps = props;
    }

    if ( superclass ) LinkProvider.__proto__ = superclass;
    LinkProvider.prototype = Object.create( superclass && superclass.prototype );
    LinkProvider.prototype.constructor = LinkProvider;
    LinkProvider.prototype.getChildContext = function getChildContext () {
        var obj;
        return ( obj = {}, obj[CONTEXT] = this.initProps, obj )
    };
    LinkProvider.prototype.render = function render () {
        return this.props.children
    };

    return LinkProvider;
}(React__default.Component));

LinkProvider.propTypes = {
    selectRoute: React.PropTypes.func.isRequired,
    rootReducer: React.PropTypes.func.isRequired,
    mapStateToURL: React.PropTypes.func,
};

LinkProvider.defaultProps = {
    mapRouteToURL: routeToURL,
};

var obj;
LinkProvider.childContextTypes = ( obj = {}, obj[CONTEXT] = React.PropTypes.object, obj );

function HashLinkProvider (ref) {
    var selectRoute = ref.selectRoute;
    var rootReducer = ref.rootReducer;
    var children = ref.children;

    return React__default.createElement(
        LinkProvider,
        { selectRoute: selectRoute, rootReducer: rootReducer, children: children, mapStateToURL: hashRouteToURL })
}

function LinkBase (ref) {
    var children = ref.children;
    var href = ref.href;
    var onNavigate = ref.onNavigate;
    var className = ref.className;
    var style = ref.style;

    var onClick = function (e) {
        e.stopPropagation();
        if (e.altKey || e.metaKey) { return }
        e.preventDefault();
        onNavigate();
    };

    return h('a', { className: className, style: style, href: href, onClick: onClick }, children)
}

var LinkConnector = reactRedux.connect(
    function (appState) { return appState; },
    function (dispatch) { return ({ dispatch: dispatch }); },
    function (appState, ref, ref$1) {
        var dispatch = ref.dispatch;
        var props = ref$1.props;
        var context = ref$1.context;

        var children = props.children;
        var action = props.action;
        var isActive = props.isActive;
        var className = props.className;
        var activeClassName = props.activeClassName;
        var style = props.style;
        var activeStyle = props.activeStyle;
        var selectRoute = context.selectRoute;
        var rootReducer = context.rootReducer;
        var mapRouteToURL = context.mapRouteToURL;

        var currentRoute = selectRoute(appState);
        var nextRoute = selectRoute(rootReducer(appState, action));
        var active = isActive(currentRoute, nextRoute);

        return {
            children: children,
            href: mapRouteToURL(nextRoute),
            onNavigate: function () { return dispatch(action); },
            className: active ? (className + " " + activeClassName) : className,
            style: active ? merge(style, activeStyle) : style,
        }
    }
)(LinkBase);

function Link (props, context) {
    return h(LinkConnector, { props: props, context: context[CONTEXT] })
}

Link.PropTypes = {
    action: React.PropTypes.object.isRequired,
    className: React.PropTypes.string,
    activeClassName: React.PropTypes.string,
    style: React.PropTypes.object,
    activeStyle: React.PropTypes.object,
    isActive: React.PropTypes.func,
};

Link.defaultProps = {
    className: '',
    activeClassName: 'active',
    isActive: defaultIsActive,
};

var obj$1;
Link.contextTypes = ( obj$1 = {}, obj$1[CONTEXT] = React.PropTypes.object, obj$1 );

function merge (a, b) {
    if (a && b) { return Object.assign({}, a, b) }
    return a || b
}

function defaultIsActive (ref, next) {
    var path = ref.path;
    var query = ref.query;

    // next path is parent or equal to current path
    for (var i = 0; i < next.path.length; i++) {
        if (!path[i] || path[i] !== next.path[i]) { return false }
    }

    // next query is subset of current query
    for (var key in next.query) {
        if (!query[key] || query[key] !== next.query[key]) { return false }
    }

    return true
}

exports.createRouteHandler = createRouteHandler;
exports.createRouterMiddleware = createRouterMiddleware;
exports.createHashRouterMiddleware = createHashRouterMiddleware;
exports.locationToRoute = locationToRoute;
exports.routeToLocation = routeToLocation;
exports.routeToURL = routeToURL;
exports.HashLinkProvider = HashLinkProvider;
exports.LinkProvider = LinkProvider;
exports.Link = Link;
exports.defaultIsActive = defaultIsActive;
